@@ -179,6 +179,9 @@ public:
     void setNameFilter(const QString& nameFilter);
     QString nameFilter() const;
 
+    void setMimeTypeFilters(const QStringList& filters);
+    QStringList mimeTypeFilters() const;
+
     struct RoleInfo
     {   QByteArray role;
         QString translation;
@@ -388,6 +391,11 @@ private:
      */
     void emitSortProgress(int resolvedCount);
 
+    /**
+     * Applies the filters set through @ref setNameFilter and @ref setMimeTypeFilters.
+     */
+    void applyFilters();
+
     /**
      * Maps the QByteArray-roles to RoleTypes and provides translation- and
      * group-contexts.
@@ -467,6 +475,7 @@ private:
     friend class KFileItemModelRolesUpdater;   // Accesses emitSortProgress() method
     friend class KFileItemModelTest;           // For unit testing
     friend class KFileItemListViewTest;        // For unit testing
+    friend class DolphinPart;                  // Accesses m_dirLister
 };
 
 inline bool KFileItemModel::isChildItem(int index) const
