@@ -20,6 +20,7 @@
 #include "dolphinsearchbox.h"
 
 #include "dolphin_searchsettings.h"
+#include "dolphinfacetswidget.h"
 #include "dolphinsearchinformation.h"
 
 #include <KIcon>
@@ -41,6 +42,7 @@
 
 #include <config-nepomuk.h>
 #ifdef HAVE_NEPOMUK
+    #include <Nepomuk/Query/AndTerm>
     #include <Nepomuk/Query/FileQuery>
     #include <Nepomuk/Query/LiteralTerm>
     #include <Nepomuk/Query/OrTerm>
@@ -65,6 +67,8 @@ DolphinSearchBox::DolphinSearchBox(QWidget* parent) :
     m_separator(0),
     m_fromHereButton(0),
     m_everywhereButton(0),
+    m_facetsToggleButton(0),
+    m_facetsWidget(0),
     m_searchPath(),
     m_readOnlyQuery(),
     m_startSearchTimer(0)
@@ -109,6 +113,10 @@ void DolphinSearchBox::setSearchPath(const KUrl& url)
     m_separator->setVisible(showSearchFromButtons);
     m_fromHereButton->setVisible(showSearchFromButtons);
     m_everywhereButton->setVisible(showSearchFromButtons);
+
+    const DolphinSearchInformation& searchInfo = DolphinSearchInformation::instance();
+    const bool hasFacetsSupport = searchInfo.isIndexingEnabled() && searchInfo.isPathIndexed(m_searchPath);
+    m_facetsWidget->setEnabled(hasFacetsSupport);
 }
 
 KUrl DolphinSearchBox::searchPath() const
@@ -191,43 +199,59 @@ void DolphinSearchBox::keyReleaseEvent(QKeyEvent* event)
     }
 }
 
-void DolphinSearchBox::emitSearchSignal()
+void DolphinSearchBox::emitSearchRequest()
 {
     m_startSearchTimer->stop();
     m_startedSearching = true;
-    emit search(m_searchInput->text());
+    emit searchRequest();
 }
 
-void DolphinSearchBox::slotSearchLocationChanged()
+void DolphinSearchBox::emitCloseRequest()
 {
-    emit searchLocationChanged(m_fromHereButton->isChecked() ? SearchFromHere : SearchEverywhere);
-}
-
-void DolphinSearchBox::slotSearchContextChanged()
-{
-    emit searchContextChanged(m_fileNameButton->isChecked() ? SearchFileName : SearchContent);
+    m_startSearchTimer->stop();
+    m_startedSearching = false;
+    emit closeRequest();
 }
 
 void DolphinSearchBox::slotConfigurationChanged()
 {
     saveSettings();
     if (m_startedSearching) {
-        emitSearchSignal();
+        emitSearchRequest();
     }
 }
 
 void DolphinSearchBox::slotSearchTextChanged(const QString& text)
 {
-    m_startSearchTimer->start();
+    if (text.isEmpty()) {
+        m_startSearchTimer->stop();
+    } else {
+        m_startSearchTimer->start();
+    }
     emit searchTextChanged(text);
 }
 
 void DolphinSearchBox::slotReturnPressed(const QString& text)
 {
-    emitSearchSignal();
+    emitSearchRequest();
     emit returnPressed(text);
 }
 
+void DolphinSearchBox::slotFacetsButtonToggled()
+{
+    const bool visible = !m_facetsWidget->isVisible();
+    m_facetsWidget->setVisible(visible);
+    SearchSettings::setShowFacetsWidget(visible);
+    updateFacetsToggleButtonIcon();
+}
+
+void DolphinSearchBox::slotFacetChanged()
+{
+    m_startedSearching = true;
+    m_startSearchTimer->stop();
+    emit searchRequest();
+}
+
 void DolphinSearchBox::initButton(QToolButton* button)
 {
     button->setAutoExclusive(true);
@@ -249,6 +273,8 @@ void DolphinSearchBox::loadSettings()
     } else {
         m_fileNameButton->setChecked(true);
     }
+
+    m_facetsWidget->setVisible(SearchSettings::showFacetsWidget());
 }
 
 void DolphinSearchBox::saveSettings()
@@ -265,7 +291,7 @@ void DolphinSearchBox::init()
     closeButton->setAutoRaise(true);
     closeButton->setIcon(KIcon("dialog-close"));
     closeButton->setToolTip(i18nc("@info:tooltip", "Quit searching"));
-    connect(closeButton, SIGNAL(clicked()), SIGNAL(closeRequest()));
+    connect(closeButton, SIGNAL(clicked()), this, SLOT(emitCloseRequest()));
 
     // Create search label
     m_searchLabel = new QLabel(this);
@@ -294,13 +320,11 @@ void DolphinSearchBox::init()
 
     m_contentButton = new QToolButton();
     m_contentButton->setText(i18nc("action:button", "Content"));
-    initButton(m_contentButton);;
+    initButton(m_contentButton);
 
     QButtonGroup* searchWhatGroup = new QButtonGroup(this);
     searchWhatGroup->addButton(m_fileNameButton);
     searchWhatGroup->addButton(m_contentButton);
-    connect(m_fileNameButton, SIGNAL(clicked()), this, SLOT(slotSearchContextChanged()));
-    connect(m_contentButton, SIGNAL(clicked()), this, SLOT(slotSearchContextChanged()));
 
     m_separator = new KSeparator(Qt::Vertical, this);
 
@@ -316,8 +340,15 @@ void DolphinSearchBox::init()
     QButtonGroup* searchLocationGroup = new QButtonGroup(this);
     searchLocationGroup->addButton(m_fromHereButton);
     searchLocationGroup->addButton(m_everywhereButton);
-    connect(m_fromHereButton, SIGNAL(clicked()), this, SLOT(slotSearchLocationChanged()));
-    connect(m_everywhereButton, SIGNAL(clicked()), this, SLOT(slotSearchLocationChanged()));
+
+    // Create "Facets" widgets
+    m_facetsToggleButton = new QToolButton(this);
+    m_facetsToggleButton->setAutoRaise(true);
+    connect(m_facetsToggleButton, SIGNAL(clicked()), this, SLOT(slotFacetsButtonToggled()));
+
+    m_facetsWidget = new DolphinFacetsWidget(this);
+    m_facetsWidget->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Maximum);
+    connect(m_facetsWidget, SIGNAL(facetChanged()), this, SLOT(slotFacetChanged()));
 
     // Apply layout for the options
     QHBoxLayout* optionsLayout = new QHBoxLayout();
@@ -328,6 +359,7 @@ void DolphinSearchBox::init()
     optionsLayout->addWidget(m_fromHereButton);
     optionsLayout->addWidget(m_everywhereButton);
     optionsLayout->addStretch(1);
+    optionsLayout->addWidget(m_facetsToggleButton);
 
     // Put the options into a QScrollArea. This prevents increasing the view width
     // in case that not enough width for the options is available.
@@ -347,6 +379,7 @@ void DolphinSearchBox::init()
     m_topLayout->setMargin(0);
     m_topLayout->addLayout(searchInputLayout);
     m_topLayout->addWidget(m_optionsScrollArea);
+    m_topLayout->addWidget(m_facetsWidget);
 
     loadSettings();
 
@@ -355,37 +388,50 @@ void DolphinSearchBox::init()
     m_startSearchTimer = new QTimer(this);
     m_startSearchTimer->setSingleShot(true);
     m_startSearchTimer->setInterval(1000);
-    connect(m_startSearchTimer, SIGNAL(timeout()), this, SLOT(emitSearchSignal()));
+    connect(m_startSearchTimer, SIGNAL(timeout()), this, SLOT(emitSearchRequest()));
 
+    updateFacetsToggleButtonIcon();
     applyReadOnlyState();
 }
 
 KUrl DolphinSearchBox::nepomukUrlForSearching() const
 {
 #ifdef HAVE_NEPOMUK
-    Nepomuk::Query::Term term;
-
+    // Create the term for the text from the input-field
+    // dependent on whether a searching for content or
+    // filename is done
     const QString text = m_searchInput->text();
-
+    Nepomuk::Query::Term searchLabelTerm;
     if (m_contentButton->isChecked()) {
         // Let Nepomuk parse the query
-        term = Nepomuk::Query::QueryParser::parseQuery(text, Nepomuk::Query::QueryParser::DetectFilenamePattern).term();
-    }
-    else {
+        searchLabelTerm = Nepomuk::Query::QueryParser::parseQuery(text, Nepomuk::Query::QueryParser::DetectFilenamePattern).term();
+    } else {
         // Search the text in the filename only
         QString regex = QRegExp::escape(text);
         regex.replace("\\*", QLatin1String(".*"));
         regex.replace("\\?", QLatin1String("."));
         regex.replace("\\", "\\\\");
-        term = Nepomuk::Query::ComparisonTerm(
-                    Nepomuk::Vocabulary::NFO::fileName(),
-                    Nepomuk::Query::LiteralTerm(regex),
-                    Nepomuk::Query::ComparisonTerm::Regexp);
+        searchLabelTerm = Nepomuk::Query::ComparisonTerm(
+                                Nepomuk::Vocabulary::NFO::fileName(),
+                                Nepomuk::Query::LiteralTerm(regex),
+                                Nepomuk::Query::ComparisonTerm::Regexp);
     }
 
+    // Get the term from the facets and merge it with the
+    // created term from the input-field.
+    Nepomuk::Query::Term facetsTerm = m_facetsWidget->facetsTerm();
+
     Nepomuk::Query::FileQuery fileQuery;
     fileQuery.setFileMode(Nepomuk::Query::FileQuery::QueryFilesAndFolders);
-    fileQuery.setTerm(term);
+    if (facetsTerm.isValid()) {
+        Nepomuk::Query::AndTerm andTerm;
+        andTerm.addSubTerm(searchLabelTerm);
+        andTerm.addSubTerm(facetsTerm);
+        fileQuery.setTerm(andTerm);
+    } else {
+        fileQuery.setTerm(searchLabelTerm);
+    }
+
     if (m_fromHereButton->isChecked()) {
         const bool recursive = true;
         fileQuery.addIncludeFolder(m_searchPath, recursive);
@@ -413,6 +459,18 @@ void DolphinSearchBox::applyReadOnlyState()
 
     m_searchInput->setVisible(!m_readOnly);
     m_optionsScrollArea->setVisible(!m_readOnly);
+
+    if (m_readOnly) {
+        m_facetsWidget->hide();
+    } else {
+        m_facetsWidget->setVisible(SearchSettings::showFacetsWidget());
+    }
+}
+
+void DolphinSearchBox::updateFacetsToggleButtonIcon()
+{
+    const bool visible = SearchSettings::showFacetsWidget();
+    m_facetsToggleButton->setIcon(KIcon(visible ? "list-remove" : "list-add"));
 }
 
 #include "dolphinsearchbox.moc"
