@@ -296,13 +296,22 @@ void PopupView::createActions()
     m_actionCollection.addAction("empty_trash", emptyTrash);
 }
 
-void PopupView::contextMenuRequest(QWidget *widget, const QPoint &screenPos)
+void PopupView::contextMenuRequest(QWidget *widget, const QPoint& screenPos)
+{
+    showContextMenu(widget, screenPos, m_selectionModel->selectedIndexes());
+}
+
+void PopupView::showContextMenu(QWidget *widget, const QPoint &screenPos, const QList<QModelIndex> &indexes)
 {
     Q_UNUSED(widget)
     // contextMenuRequest is only called from the icon view, which is created in init()
     // which mean m_model should always be initialized
     Q_ASSERT(m_model);
 
+    if (indexes.isEmpty()) {
+        return;
+    }
+
     if (m_actionCollection.isEmpty()) {
         createActions();
     }
@@ -327,10 +336,12 @@ void PopupView::contextMenuRequest(QWidget *widget, const QPoint &screenPos)
         }
     }
 
-    QAction* pasteTo = m_actionCollection.action("pasteto");
+    QAction *pasteTo = m_actionCollection.action("pasteto");
     if (pasteTo) {
-        pasteTo->setEnabled(m_actionCollection.action("paste")->isEnabled());
-        pasteTo->setText(m_actionCollection.action("paste")->text());
+        if (QAction *paste = m_actionCollection.action("paste")) {
+            pasteTo->setEnabled(paste->isEnabled());
+            pasteTo->setText(paste->text());
+        }
     }
 
     QList<QAction*> editActions;
